"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const graphql_1 = require("graphql");
const graphql_transformer_core_1 = require("graphql-transformer-core");
const graphql_transformer_common_1 = require("graphql-transformer-common");
const graphql_mapping_template_1 = require("graphql-mapping-template");
class AssertTransformer extends graphql_transformer_core_1.Transformer {
    constructor() {
        super('AssertTransformer', graphql_transformer_core_1.gql `
        directive @assert(condition: String!, message: String, type: String) on FIELD_DEFINITION
      `);
        this.field = (parent, definition, directive, ctx) => {
            if (parent.kind === graphql_1.Kind.INTERFACE_TYPE_DEFINITION) {
                throw new graphql_transformer_core_1.InvalidDirectiveError(`The @assert directive cannot be placed on an interface's field. See ${parent.name.value}${definition.name.value}`);
            }
            // Validation - @model required
            this.validateParentModelDirective(parent);
            const { condition, message, type } = graphql_transformer_core_1.getDirectiveArguments(directive);
            // Generate the VTL code block
            const typeName = parent.name.value;
            const fieldName = definition.name.value;
            const validationExpression = this.generateValidationExpression(fieldName, condition, message, type);
            const vtlCode = graphql_mapping_template_1.printBlock(`Assert validation for "${fieldName}" (${condition})`)(validationExpression);
            // Update create and update mutations
            const createResolverResourceId = graphql_transformer_common_1.ResolverResourceIDs.DynamoDBCreateResolverResourceID(typeName);
            this.updateResolver(ctx, createResolverResourceId, vtlCode);
            const updateResolverResourceId = graphql_transformer_common_1.ResolverResourceIDs.DynamoDBUpdateResolverResourceID(typeName);
            this.updateResolver(ctx, updateResolverResourceId, vtlCode);
        };
        this.validateParentModelDirective = (type) => {
            const directive = type.directives.find((d) => d.name.value === 'model');
            if (!directive) {
                throw new Error(`@assert directive can only be used on types with @model directive.`);
            }
        };
        this.quote = (s) => {
            return `'${s.replace(/'/g, "''")}'`;
        };
        this.generateValidationExpression = (fieldName, condition, message, type) => {
            const name = this.quote(fieldName);
            const cond = this.quote(condition);
            const val = `$ctx.args.input.${fieldName}`;
            const cls = `${val}.getClass()`;
            const cond1 = condition.replace(/\B\.\B/g, val).replace(/\B\./g, `${val}.`);
            const msg = this.quote(message || 'Input assertion error');
            const typ = this.quote(type || 'AssertionError');
            return graphql_mapping_template_1.iff(graphql_mapping_template_1.and([graphql_mapping_template_1.not(graphql_mapping_template_1.raw(`$util.isNull(${val})`)), graphql_mapping_template_1.not(graphql_mapping_template_1.parens(graphql_mapping_template_1.raw(cond1)))]), graphql_mapping_template_1.ref(`util.error(
                    ${msg},
                    ${typ},
                    null,
                    {
                        "type": "assertion",
                        "condition": ${cond},
                        "fieldName": ${name},
                        "fieldValue": ${val},
                        "fieldClass": ${cls}
                    })`));
        };
        this.updateResolver = (ctx, resolverResourceId, code) => {
            const resolver = ctx.getResource(resolverResourceId);
            if (resolver) {
                const templateParts = [code, resolver.Properties.RequestMappingTemplate];
                resolver.Properties.RequestMappingTemplate = templateParts.join('\n\n');
                ctx.setResource(resolverResourceId, resolver);
            }
        };
    }
}
exports.AssertTransformer = AssertTransformer;
//# sourceMappingURL=AssertTransformer.js.map